@@ -1,8 +1,6 @@
 /* WORequest+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +33,9 @@
 - (BOOL) isICal;
 - (BOOL) isICal4;
 - (BOOL) isMacOSXAddressBookApp;
-- (BOOL) isIPhoneAddressBookApp;
+- (BOOL)isMacOSXCalendarApp;
+- (BOOL)isIPhoneAddressBookApp;
+- (BOOL) isAndroid;
 
 @end
 
