@@ -23,6 +23,10 @@
       $mdDialog.hide();
     };
 
+    this.changed = function (d) {
+      console.log(d);
+    };
+
     this.highPriority = function () {
       return (this.component &&
               this.component.priority &&
@@ -216,6 +220,8 @@
       this.categories = {};
       this.showRecurrenceEditor = this.component.$hasCustomRepeat;
       this.showAttendeesEditor = this.component.attendees && this.component.attendees.length;
+      this.isFullscreen = false;
+      this.originalModalCancel = $mdDialog.cancel;
 
       if (this.component.type == 'appointment') {
         this.component.initAttendees();
@@ -245,8 +251,53 @@
 
       dayStartTime = parseInt(Preferences.defaults.SOGoDayStartTime);
       dayEndTime = parseInt(Preferences.defaults.SOGoDayEndTime);
+
+      this.originalHash = this.hash(this.component);
+      $mdDialog.cancel = function () {
+        if (vm.originalHash === vm.hash(vm.component)  || confirm(l('You have modified data unsaved. Do you want to close popup and loose data ?'))) {
+          $mdDialog.cancel = vm.originalModalCancel;
+          return vm.originalModalCancel();
+        }
+      };
     };
 
+    this.hash = function (data) {
+      var hash = 0, i, chr, edata, json;
+      edata = {
+        repeat: data.repeat,
+        pid: data.pid,
+        destinationCalendar: data.destinationCalendar,
+        classification: data.classification,
+        categories: data.categories,
+        alarm: data.alarm,
+        summary: data.summary,
+        status: data.status,
+        organizer: data.organizer,
+        location: data.location,
+        isAllDay: data.isAllDay,
+        comment: data.comment,
+        attendees: data.attendees
+      };
+      if (edata.organizer && edata.organizer.freebusy) {
+        edata.organizer.freebusy = {};
+      }
+      if (edata.attendees) {
+        for (i = 0; i < edata.attendees.length; i++) {
+          edata.attendees[i].freebusy = {};
+        }
+      }
+      json = JSON.stringify(edata);
+
+      if (json.length === 0) return hash;
+      for (i = 0; i < json.length; i++) {
+        chr = json.charCodeAt(i);
+        hash = ((hash << 5) - hash) + chr;
+        hash |= 0;
+      }
+
+      return hash;
+    }
+
     this.addAttachUrl = function () {
       var i = this.component.addAttachUrl('');
       focus('attachUrl_' + i);
@@ -279,9 +330,14 @@
     };
 
     this.destinationCalendars = function () {
-      if (this.component.isErasable)
+      if (this.component && this.component.isNew)
+        // New component, return all writable calendars
+        return Calendar.$findAll(null, true);
+      else if (this.component && this.component.isErasable)
+        // Movable component, return all writable calendars including current one
         return Calendar.$findAll(null, true, this.component.pid);
       else
+        // Component can't be moved
         return [Calendar.$get(this.component.pid)];
     };
 
@@ -291,6 +347,10 @@
         this.component.$attendees.initOrganizer(Calendar.$get(this.component.destinationCalendar));
     };
 
+    this.toggleFullscreen = function() {
+      vm.isFullscreen = !vm.isFullscreen;
+    }
+
     // Autocomplete cards for attendees
     this.cardFilter = function ($query) {
       return AddressBook.$filterAll($query);
@@ -484,6 +544,7 @@
             .then(function(data) {
               $rootScope.$emit('calendars:list');
               Preferences.getAlarms();
+              $mdDialog.cancel = vm.originalModalCancel;
               $mdDialog.hide();
             }, function(response) {
               if (response.status == CalendarSettings.ConflictHTTPErrorCode)
@@ -501,6 +562,14 @@
     };
 
     this.cancel = function (form) {
+      if (vm.originalHash === vm.hash(vm.component) || confirm(l('You have modified data unsaved. Do you want to close popup and loose data ?'))) {
+        $mdDialog.cancel = vm.originalModalCancel;
+      } else {
+        return;
+      }
+
+      $mdDialog.hide();
+      
       this.reset(form);
       if (this.component.isNew) {
         // Cancelling the creation of a component
